/*
** Module   : XNodeRefImpl
** Abstract : DOM XML Node reference resource type implementation.
**
** Copyright (c) 2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- -----------------------Description------------------------
** 001 EVL 20130204 Created initial version. Major rework to use w3c backend.
*/

package com.goldencode.p2j.xml;

import java.util.*;
import org.w3c.dom.*;
import com.goldencode.p2j.util.*;

/**
 * Implementation of the X-Node-Reference specific features of the Progress
 * DOM XML support objects.
 */
public class XNodeRefImpl
extends XEntityImpl
implements XNodeRef
{
   /**
    * Default constructor.
    */
   public XNodeRefImpl()
   {
      super();
   }

   /**
    * Special constructor accepting the namspace URI specification to make
    * another node reference case.
    * 
    * @param    element
    *           The Element associated with this node.
    */
   public XNodeRefImpl(Element element)
   {
      super(element);
   }

   // Attributes
   /**
    * Getting the attribute names available for curreny node reference.
    * 
    * @return   The comma separated character string of all available
    *           attributed.
    */
   public character getAttributeNames()
   {
      StringBuilder sb = new StringBuilder();
      NamedNodeMap nnm = getNode().getAttributes();
      int iSize = nnm.getLength();
      for (int i = 0; i < iSize; i++)
      {
         sb.append(((Attr)nnm.item(i)).getName());
         if (i < iSize - 1)
         {
            sb.append(",");
         }
      }   
      
      return new character(sb.toString());
   }
   
   /**
    * Getting the Docunment the current reference belongs to.
    * 
    * @return   The handle of the Document containing tne curreny reference..
    */
   public handle getOwnerDocument()
   {
      return getHandle();
   }
   
   // Methods
   /**
    * Deleting the internals of the current X-Node-Reference.
    * 
    * @return   The <code>true</code> in case of success
    *           <code>false</code> otherwise.
    */
   public logical deleteNode()
   {
      clear();
      return new logical(true);
   }
   
   /**
    * Getting the attribute value for a given attribute name.
    * 
    * @param    name
    *           The name of the attribute to return.
    *
    * @return   The comma separated character string of all available
    *           attributed.
    */
   public character getAttribute(String name)
   {
      return getAttribute(new character(name));
   }
   
   /**
    * Getting the attribute value for a given attribute name.
    * 
    * @param    name
    *           The name of the attribute to return.
    *
    * @return   The comma separated character string of all available
    *           attributed.
    */
   public character getAttribute(character name)
   {
      if (name.isUnknown())
      {
         return new character();
      }
      else
      {
         return new character(getNode().getAttribute(name.toStringMessage()));
      }
   }
   
   /**
    * Setting the attribute value for a given attribute name.
    * 
    * @param    name
    *           The name of the attribute to change.
    * @param    value
    *           The new value for the given attribute name.
    * 
    * @return   The <code>true</code> in case of success
    *           <code>false</code> otherwise.
    */
   public logical setAttribute(String name, String value)
   {
      return setAttribute(new character(name), new character(value));
   }
   
   /**
    * Setting the attribute value for a given attribute name.
    * 
    * @param    name
    *           The name of the attribute to change.
    * @param    value
    *           The new value for the given attribute name.
    * 
    * @return   The <code>true</code> in case of success
    *           <code>false</code> otherwise.
    */
   public logical setAttribute(String name, character value)
   {
      return setAttribute(new character(name), value);
   }
   
   /**
    * Setting the attribute value for a given attribute name.
    * 
    * @param    name
    *           The name of the attribute to change.
    * @param    value
    *           The new value for the given attribute name.
    * 
    * @return   The <code>true</code> in case of success
    *           <code>false</code> otherwise.
    */
   public logical setAttribute(character name, String value)
   {
      return setAttribute(name, new character(value));
   }
   
   /**
    * Setting the attribute value for a given attribute name.
    * 
    * @param    name
    *           The name of the attribute to change.
    * @param    value
    *           The new value for the given attribute name.
    * 
    * @return   The <code>true</code> in case of success
    *           <code>false</code> otherwise.
    */
   public logical setAttribute(character name, character value)
   {
      if (name.isUnknown() || value.isUnknown())
      {
         return new logical(false);
      }
      else
      {
         getNode().setAttribute(name.toStringMessage(),
                                value.toStringMessage());
         return new logical(false);
      }
   }
}
